@@ -5,58 +5,81 @@ import logging
 import sys
 from os import path
 import argparse
+
+import abstract_instrument_interface
 from pySRSLockin.driver import SRS_Lockin
-from pySRSLockin.plots import PlotObject
-from pySRSLockin.abstract_interface import abstract_interface, abstract_gui
 
 graphics_dir = path.join(path.dirname(__file__), 'graphics')
 
-class interface(abstract_interface):
+class interface(abstract_instrument_interface.abstract_interface):
     """
-    Create a high-level interface with the device, and provide a GUI
-
+    Create a high-level interface with the device, and act as a connection between the low-level
+    interface (i.e. the driver) and the gui.
+    Several general-purpose attributes and methods are defined in the class abstract_interface defined in abstract_instrument_interface
     ...
 
-    Attributes
+    Attributes specific for this class (see the abstract class abstract_instrument_interface.abstract_interface for general attributes)
     ----------
-    output : dict
-        a dictionary containing the output data created by this interface
-    app : Qt.QApplication
-        The pyqt5 QApplication() object
-    mainwindow  : Qt.QWidget
-        The main window of the Qt application
-    instrument : SRS_Lockin
-        a driver object, for the low-level communication with the device
-    gui : gui()
-        instance of the class gui(), containing all guis element and basic function to set different states
+    instrument
+        Instance of driver.pySRSLockin
+    connected_device_name : str
+        Name of the physical device currently connected to this interface 
+    continuous_read : bool 
+        When this is set to True, the data from device are acquired continuosly at the rate set by refresh_time
+    refresh_time : float, 
+        The time interval (in seconds) between consecutive reeading from the device driver (default = 0.2)
+    stored_data : list
+        List used to store data acquired by this interface
+
+
+    Methods defined in this class (see the abstract class abstract_instrument_interface.abstract_interface for general methods)
+    -------
+    refresh_list_devices()
+        Get a list of compatible devices from the driver. Stores them in self.list_devices, and populates the combobox in the GUI.
+    connect_device(device_full_name)
+        Connect to the device identified by device_full_name
+    disconnect_device()
+        Disconnect the currently connected device
+    close()
+        Closes this interface, close plot window (if any was open), and calls the close() method of the parent class, which typically calls the disconnect_device method
+    
+    set_disconnected_state()
+        
+    set_connecting_state()
+    
+    set_connected_state()
+    
+    set_refresh_time(refresh_time)
+    
+    [TO FINISH]
+
     """
     output = {'X':0,'Y':0,'mag':0,'theta':0} #We define this also as class variable, to make it possible to see which data is produced by this interface without having to create an object
 
     def __init__(self, **kwargs):
         self.output = {'X':0,'Y':0,'mag':0,'theta':0} #We define this also as class variable, to make it possible to see which data is produced by this interface without having to create an object
-        self.continuous_read = False #When this is set to True, the data from device are acquired continuosly at the rate set by self.refresh_time
-        self.refresh_time = 0.2 #default refresh rate, in seconds
-        self.sensitivity_auto = False #Boolean, true if software-controlled automatic sensitivity is enabled
+        
+        ### Default values of settings (might be overlapped by settings saved in .json files later)
+        self.settings = {   'refresh_time': 0.2,        #default refresh rate, in seconds
+                            'sensitivity_auto': True    #Boolean, true if software-controlled automatic sensitivity is enabled
+                            }
+
+        self.list_devices = []          #list of devices found
+        self.continuous_read = False #When this is set to True, the data from device are acquired continuosly at the rate set by self.settings['refresh_time']
         self.stored_data = {'X':[],'Y':[],'mag':[],'theta':[]}  # List used to store recorded data
         self.plot_window = None # QWidget object of the widget (i.e. floating window) that will contain the plot
         self.plot_object = None # PlotObject object of the plot where self.store_powers is plotted
         self.connected_device_name = ''
         
         self.instrument = SRS_Lockin() 
+        ###
+        self.gui_class = gui
+        ###
         super().__init__(**kwargs)
 
-    def create_gui(self,parent,plot=False): 
-        '''
-            parent, QWidget        
-                a QWidget (or QMainWindow) object that will be the parent for the gui of this device.
-                When this script is ran on its own (i.e. not within another gui) self.mainwindow and parent are actually the same object
-            plot, boolean
-                If set true, the GUI also generates a plot object (and a button to show/hide the plot) to plot the content of the self.stored_data object
-        '''
-        self.gui = gui(self,parent,plot)
-        self.gui.initialize() 
-
-### Functions to talk with low-level driver
+############################################################
+### Functions to interface the GUI and low-level driver
+############################################################
 
     def refresh_list_devices(self):
         '''
@@ -90,7 +113,6 @@ class interface(abstract_interface):
             self.logger.info(f"Connected to device {device_name}.")
             self.connected_device_name = device_name
             self.set_connected_state()
-
             self.start_reading()
         else: #If connection was not successful
             self.logger.error(f"Error: {Msg}")
@@ -109,7 +131,14 @@ class interface(abstract_interface):
             self.set_disconnected_state()
         else: #If disconnection was not successful
             self.logger.error(f"Error: {Msg}")
-            
+            self.set_disconnected_state() #When disconnection is not succeful, it is typically because the device alredy lost connection
+                                          #for some reason. In this case, it is still useful to have all widgets reset to disconnected state  
+    def close(self,**kwargs):
+        if hasattr(self.gui,'plot_window'):
+            if self.gui.plot_window:
+                self.gui.plot_window.close()
+        super().close(**kwargs)      
+                                          
     def set_disconnected_state(self):
         self.gui.set_disconnected_state()
 
@@ -181,15 +210,19 @@ class interface(abstract_interface):
         self.gui.combo_Sensitivities.setCurrentIndex(self.sensitivity)
         self.logger.info(f"Current sensitivity is {self.sensitivities[self.sensitivity]} (index={self.sensitivity})...") 
         
-    def set_sensitivity(self,s): 
+    def set_sensitivity(self,s, log = True): 
+        #when log = False, the message regarding the change of sensitivities is not added to the logs
+        #this is useful to avoid excessively polluting the logs when sensitivty is changed very often automatically
         if s == self.sensitivity:
             return
-        self.logger.info(f"Setting sensitivity to {self.sensitivities[s]}  (index={s})...")
+        if log:
+            self.logger.info(f"Setting sensitivity to {self.sensitivities[s]}  (index={s})...")
         try: 
             self.instrument.sensitivity = s #set sensitivity
             snew = self.instrument.sensitivity #read again sensitivity to confirm that it changed
             if snew == s:
-                self.logger.info(f"Sensitivity changed to {self.sensitivities[s]} (old value was {self.sensitivities[self.sensitivity]}).")
+                if log:
+                    self.logger.info(f"Sensitivity changed to {self.sensitivities[s]} (old value was {self.sensitivities[self.sensitivity]}).")
                 self.sensitivity = s
             else:
                 self.logger.error(f"The driver did not raise any error, but it was not possible to change the sensitivity.")
@@ -199,7 +232,7 @@ class interface(abstract_interface):
         return True
 
     def set_sensitivty_auto(self,status):
-        self.sensitivity_auto = status
+        self.settings['sensitivity_auto'] = status
         if status==True:
             self.read_sensitivity() #make sure to read the current value of sensitivity
 
@@ -216,30 +249,30 @@ class interface(abstract_interface):
         if (current_sensitivity_value*min_bound) < current_mag < (current_sensitivity_value*max_bound):
             return False #In this case we don't need to change the sensitivity
         else:
-            self.logger.info(f"Need to adjust sensitivity.")
+            #self.logger.info(f"Need to adjust sensitivity.")
             target_sensitivity = current_mag / max_bound
             #Look for smallest sensitivity larger than target_sensitivity
             index_new_sensitivity = self.sensitivities.index (
                                                                 min ([i for i in  self.sensitivities if i > target_sensitivity] )
                                                                 )
-            return self.set_sensitivity(index_new_sensitivity)
+            return self.set_sensitivity(index_new_sensitivity,log = False)
 
     def set_refresh_time(self, refresh_time):
         try: 
             refresh_time = float(refresh_time)
-            if self.refresh_time == refresh_time: #in this case the number in the refresh time edit box is the same as the refresh time currently stored
+            if self.settings['refresh_time'] == refresh_time: #in this case the number in the refresh time edit box is the same as the refresh time currently stored
                 return True
         except ValueError:
             self.logger.error(f"The refresh time must be a valid number.")
-            self.gui.edit_RefreshTime.setText(f"{self.refresh_time:.3f}")
+            self.gui.edit_RefreshTime.setText(f"{self.settings['refresh_time']:.3f}")
             return False
         if refresh_time < 0.001:
             self.logger.error(f"The refresh time must be positive and >= 1ms.")
-            self.gui.edit_RefreshTime.setText(f"{self.refresh_time:.3f}")
+            self.gui.edit_RefreshTime.setText(f"{self.settings['refresh_time']:.3f}")
             return False
         self.logger.info(f"The refresh time is now {refresh_time} s.")
-        self.refresh_time = refresh_time
-        self.gui.edit_RefreshTime.setText(f"{self.refresh_time:.3f}")
+        self.settings['refresh_time'] = refresh_time
+        self.gui.edit_RefreshTime.setText(f"{self.settings['refresh_time']:.3f}")
         return True
    
     def start_reading(self):
@@ -281,7 +314,7 @@ class interface(abstract_interface):
         If we are continuosly acquiring (i.e. if self.ContinuousRead = 1) then:
             1) Reads data from device object and stores it in the self.output dictionary
             2) Update the value of the corresponding edits widgets in the GUI
-            3) Call itself after a time given by self.refresh_time
+            3) Call itself after a time given by self.settings['refresh_time']
         '''
         #self.plot_object.data.setData(list(range(1, len(self.stored_powers)+1)), self.stored_powers) #This line is executed even when self.continuous_read == False, to make sure that plot gets cleared when user press the stop button
         if(self.continuous_read == True):
@@ -301,49 +334,64 @@ class interface(abstract_interface):
                 except:
                     self.gui.edit_Output[k].setText(f"nan")
 
-            if self.sensitivity_auto == True:
+            if self.settings['sensitivity_auto'] == True:
                 self.adjust_sensitivity_auto()
 
-            QtCore.QTimer.singleShot(int(self.refresh_time*1e3), self.update)
+            QtCore.QTimer.singleShot(int(self.settings['refresh_time']*1e3), self.update)
            
         return
 
-class gui(abstract_gui):
-    def __init__(self,interface,parent,plot=False):
+############################################################
+### END Functions to interface the GUI and low-level driver
+############################################################
+
+class gui(abstract_instrument_interface.abstract_gui):
+    def __init__(self,interface,parent):
+        """
+        Attributes specific for this class (see the abstract class abstract_instrument_interface.abstract_gui for general attributes)
+        ----------
+        ...
+        """
         super().__init__(interface,parent)
 
         self.widgets_enabled_when_connected = []     #The widgets in this list will only be enabled when the interface has succesfully connected to a powermeter
         self.widgets_enabled_when_disconnected = []  #The widgets in this list will only be enabled when the interface is not connected to a powermeter
         
+    def initialize(self):
+        self.create_widgets()
+
+        ### SET INITIAL STATE OF WIDGETS
+        self.click_button_refresh_list_devices()    #By calling this method, as soon as the gui is created we also look for devices
+        self.set_disconnected_state()               #When GUI is created, all widgets are set to the "Disconnected" state
+        if self.interface.settings['sensitivity_auto']:
+            self.box_SensitivityAuto.setChecked(True)
+        ###
+
+        self.connect_widgets_events_to_functions()
+
+        ### Call the initialize method of the parent class
+        super().initialize()
+
+    def create_widgets(self):
         hbox1 = Qt.QHBoxLayout()
         self.label_DeviceList = Qt.QLabel("Devices: ")
         self.combo_Devices = Qt.QComboBox()
-        #self.combo_Devices.setFixedWidth(300)
         self.button_RefreshDeviceList = Qt.QPushButton("")
         self.button_RefreshDeviceList.setIcon(QtGui.QIcon(path.join(graphics_dir,'refresh.png')))
-        self.button_RefreshDeviceList.clicked.connect(self.click_button_refresh_list_devices)
         hbox1.addWidget(self.label_DeviceList)
         hbox1.addWidget(self.combo_Devices,stretch=1)
         hbox1.addWidget(self.button_RefreshDeviceList)
-        #hbox1.addStretch(1)
 
         hbox2 = Qt.QHBoxLayout()
         self.button_ConnectDevice = Qt.QPushButton("Connect")
-        self.button_ConnectDevice.clicked.connect(self.click_button_connect_disconnect)
         self.button_SetAutoScale = Qt.QPushButton("Set Auto Scale")
-        self.button_SetAutoScale.clicked.connect(self.click_button_set_auto_scale)
         self.label_TimeConstants = Qt.QLabel("Time constant (s): ")
         self.combo_TimeConstants = Qt.QComboBox()
-        self.combo_TimeConstants.activated.connect(self.click_combo_time_constants)
 
         self.label_Sensitivities = Qt.QLabel("Sensitivity (V): ")
         self.combo_Sensitivities = Qt.QComboBox()
-        self.combo_Sensitivities.activated.connect(self.click_combo_sensitivities)
 
         self.box_SensitivityAuto = Qt.QCheckBox("Auto")
-        if self.interface.sensitivity_auto:
-            self.box_SensitivityAuto.setChecked(True)
-        self.box_SensitivityAuto.stateChanged.connect(self.click_box_SensitivityAuto)
         self.box_SensitivityAuto.setToolTip('Enables a software-controlled adjustement of sensitivity.')
 
         hbox2.addWidget(self.button_ConnectDevice)
@@ -353,7 +401,6 @@ class gui(abstract_gui):
         hbox2.addWidget(self.label_Sensitivities)
         hbox2.addWidget(self.combo_Sensitivities,stretch=1)
         hbox2.addWidget(self.box_SensitivityAuto)
-        #hbox2.addStretch(1)
 
         hbox3 = Qt.QHBoxLayout()
         hbox3_vbox1 = Qt.QVBoxLayout()
@@ -361,28 +408,24 @@ class gui(abstract_gui):
         self.button_StartPauseReading = Qt.QPushButton("")
         self.button_StartPauseReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'play.png')))
         self.button_StartPauseReading.setToolTip('Start or pause the reading from the device. The previous data points are not discarded when pausing.') 
-        self.button_StartPauseReading.clicked.connect(self.click_button_StartPauseReading)
-        if plot:
-            self.button_StopReading = Qt.QPushButton("")
-            self.button_StopReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'stop.png')))
-            self.button_StopReading.setToolTip('Stop the reading from the device. All previous data points are discarded.') 
-            self.button_StopReading.clicked.connect(self.click_button_StopReading)
-        else:
-            self.button_StopReading = None
+        #if plot:
+        #    self.button_StopReading = Qt.QPushButton("")
+        #    self.button_StopReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'stop.png')))
+        #    self.button_StopReading.setToolTip('Stop the reading from the device. All previous data points are discarded.') 
+        #    self.button_StopReading.clicked.connect(self.click_button_StopReading)
+        #else:
+        #    self.button_StopReading = None
         hbox3_vbox1_hbox1 = Qt.QHBoxLayout()
         hbox3_vbox1_hbox1.addWidget(self.button_StartPauseReading)  
-        if plot:
-            hbox3_vbox1_hbox1.addWidget(self.button_StopReading)
-        
+        #if plot:
+        #    hbox3_vbox1_hbox1.addWidget(self.button_StopReading)
         
         self.label_RefreshTime = Qt.QLabel("Refresh time (s): ")
         self.label_RefreshTime.setToolTip('Specifies how often data is read from the device (Minimum value = 0.001 s).') 
         self.edit_RefreshTime  = Qt.QLineEdit()
-        self.edit_RefreshTime.setText(f"{self.interface.refresh_time:.3f}")
+        self.edit_RefreshTime.setText(f"{self.interface.settings['refresh_time']:.3f}")
         self.edit_RefreshTime.setToolTip('Specifies how often data is read from the device (Minimum value = 0.001 s).') 
-        self.edit_RefreshTime.returnPressed.connect(self.press_enter_refresh_time)
         self.edit_RefreshTime.setAlignment(QtCore.Qt.AlignRight)
-        #self.edit_RefreshTime.setMaximumWidth(50)
         hbox3_vbox1_hbox2 = Qt.QHBoxLayout()
         hbox3_vbox1_hbox2.addWidget(self.label_RefreshTime)  
         hbox3_vbox1_hbox2.addWidget(self.edit_RefreshTime,stretch=1)
@@ -397,7 +440,7 @@ class gui(abstract_gui):
         self.label_Output = dict()    
         self.edit_Output = dict() 
         self.widget_Output = dict()     
-        for k in self.interface.output.keys():		
+        for k in self.interface.output.keys():	#Create display textbox for each of the output data of this interface	
             self.label_Output[k] = Qt.QLabel(f"{k}: ")
             self.label_Output[k].setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignCenter)
             self.label_Output[k].setFont(fontLabel)
@@ -421,18 +464,10 @@ class gui(abstract_gui):
         self.layout_ContainerOutputs.setContentsMargins(0, 0, 0, 0)
         self.widget_ContainerOutputs = Qt.QWidget()
         self.widget_ContainerOutputs.setLayout(self.layout_ContainerOutputs)
-        #self.button_ShowHidePlot = Qt.QPushButton("Show/Hide Plot")
-        #self.button_StopReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'stop.png')))
-        #self.button_ShowHidePlot.setToolTip('Show/Hide Plot.') 
-        #self.button_ShowHidePlot.clicked.connect(self.click_button_ShowHidePlot)
 
         hbox3.addLayout(hbox3_vbox1)
-        #hbox3.addStretch(1)
         hbox3.addWidget(self.widget_ContainerOutputs,stretch=1)
-        #hbox3.addWidget(self.button_ShowHidePlot)
-        #hbox3.addStretch(1)
-        #hbox3.setAlignment(QtCore.Qt.AlignTop)
-                
+     
         self.container = Qt.QVBoxLayout()
         self.container.addLayout(hbox1)  
         self.container.addLayout(hbox2)  
@@ -444,17 +479,22 @@ class gui(abstract_gui):
                                                self.label_Sensitivities,
                                                self.combo_Sensitivities,
                                                self.button_StartPauseReading,
-                                               self.button_StopReading,
+                                               #self.button_StopReading,
                                                self.box_SensitivityAuto]
 
         self.widgets_enabled_when_disconnected = [self.combo_Devices , 
                                                   self.button_RefreshDeviceList]
     
-    def initialize(self):
-        super().initialize()
-        self.click_button_refresh_list_devices()    #By calling this method, as soon as the gui is created we also look for devices
-        self.set_disconnected_state()               #When GUI is created, all widgets are set to the "Disconnected" state
 
+    def connect_widgets_events_to_functions(self):
+        self.button_RefreshDeviceList.clicked.connect(self.click_button_refresh_list_devices)
+        self.button_ConnectDevice.clicked.connect(self.click_button_connect_disconnect)
+        self.button_SetAutoScale.clicked.connect(self.click_button_set_auto_scale)
+        self.combo_TimeConstants.activated.connect(self.click_combo_time_constants)
+        self.combo_Sensitivities.activated.connect(self.click_combo_sensitivities)
+        self.box_SensitivityAuto.stateChanged.connect(self.click_box_SensitivityAuto)
+        self.button_StartPauseReading.clicked.connect(self.click_button_StartPauseReading)
+        self.edit_RefreshTime.returnPressed.connect(self.press_enter_refresh_time)
         
     def set_disconnected_state(self):
         self.disable_widget(self.widgets_enabled_when_connected)
@@ -478,7 +518,9 @@ class gui(abstract_gui):
     def set_stopped_state(self):    
         self.button_StartPauseReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'play.png')))
 
-    ### GUI Events Functions
+############################################################
+### GUI Events Functions
+############################################################
 
     def click_button_refresh_list_devices(self):
         self.interface.refresh_list_devices()
@@ -520,7 +562,9 @@ class gui(abstract_gui):
     def press_enter_refresh_time(self):
         return self.interface.set_refresh_time(self.edit_RefreshTime.text())
 
+############################################################
 ### END GUI Events Functions
+############################################################
 
  
 class MainWindow(Qt.QWidget):
@@ -531,7 +575,7 @@ class MainWindow(Qt.QWidget):
         # self.setCentralWidget(self.container)
     def closeEvent(self, event):
         if self.child:
-            self.child.close()
+            pass #self.child.close()
 
 def main():
     parser = argparse.ArgumentParser(description = "",epilog = "")
@@ -543,8 +587,7 @@ def main():
     Interface = interface(app=app,mainwindow=window) #In this case window is both the MainWindow and the parent of the gui
     Interface.verbose = not(args.decrease_verbose)
     app.aboutToQuit.connect(Interface.close) 
-    Interface.create_gui(window,plot=False)
-    #Interface.set_trigger(lambda : print('test'),delay=1000)
+    Interface.create_gui(window)
     window.show()
     app.exec()# Start the event loop.
 
